/**
 * This script emulates Firefox's "Report a broken website" functionality
 * Some parts of the code have been taken from:
 * http://mxr.mozilla.org/mozilla-central/source/extensions/reporter/resources/content/reporter/reportWizard.js
 * in order to reproduce the desired behavior of the Reporter
 */
REPORTER_ALERTS_FORM_SUBMIT_START = "reporter-form-submit-start";
REPORTER_ALERTS_FORM_SUBMIT_PROGRESS = "reporter-form-submit-progress";
REPORTER_ALERTS_FORM_SUBMIT_ERROR = "reporter-form-submit-error";
REPORTER_ALERTS_FORM_SUBMIT_END = "reporter-form-submit-end";

if ("undefined" == typeof(ReporterForFennec))
{
  var ReporterForFennec = {};
};


ReporterForFennec = {

  // Globals
  gParamLanguage: navigator.language,
  gRMOvers: "0.3", // Do not touch without contacting reporter admin!
  gParamUserAgent: navigator.userAgent,
  gParamOSCPU: navigator.oscpu,
  gParamPlatform: navigator.platform,

  gParamURL: "",
  gCharset: "",
  gParamDescription: "",
  gParamProblemType: "",
  gParamBehindLogin: "",
  gParamEmail: "",
  gParamBuildConfig: "",
  gParamGecko: "",
  gPrefBranch: null,
  gStatusIndicator: "",

  _reporterAndSearchEnginesContainer: {},
  _reporterCreateReportContainer: {},
  _reporterProblemCategoriesList: {},
  _reporterComments: {},
  _reporterCreateReportButton: {},
  _identityPopup: {},
  _privacyPolicyURL: {},
  _stringBundle: {},
  _notifier: {},
  _observer: {},
  _oldShowFunction: {},
  _urlMaxSize: 50,
  _isReporterPopUpShown: false,

  Cc : Components.classes,
  Ci : Components.interfaces,

  init : function ()
  {
    this._cacheElements();

    /**
     * we need to rebuild the search engines panel in the Website Id panel
     * in order to add our new "Report a problem button"
     */
    //get the search-container element
    var oldSearchContainer = document.getElementById('search-container');

    //delete the original search-container element
    var parentForSearchContainer = document.getElementById('identity-container');
    parentForSearchContainer.removeChild(oldSearchContainer);

    //put the search-container element inside our new panel
    var searchContainerVbox = document.getElementById('search-container-vbox');
    searchContainerVbox.appendChild(oldSearchContainer);

    this.overrideIdentityShow();
  },

  unload: function ()
  {
    this._observer.removeObserver(this, REPORTER_ALERTS_FORM_SUBMIT_START);
    this._observer.removeObserver(this, REPORTER_ALERTS_FORM_SUBMIT_PROGRESS);
    this._observer.removeObserver(this, REPORTER_ALERTS_FORM_SUBMIT_ERROR);
    this._observer.removeObserver(this, REPORTER_ALERTS_FORM_SUBMIT_END);
  },

  /**
   * result: preloads visual elements, notifiers, bundles and sets properties for elements
   */
  _cacheElements: function()
  {
    this._reporterAndSearchEnginesContainer = document.getElementById('reporter-and-searchEngines-container');
    this._reporterCreateReportContainer = document.getElementById('reporter-create-report-container');
    this._reporterProblemCategoriesList = document.getElementById('reporter-problem-categories-list');
    this._reporterComments = document.getElementById('reporter-comments');
    this._reporterCreateReportButton = document.getElementById('reporter-create-report-button');
    this._identityPopupContainer = document.getElementById('identity-popup-container');
    this._notifier = Cc["@mozilla.org/alerts-service;1"].getService(Ci.nsIAlertsService);
    this._stringBundle = document.getElementById('reporter-strings');
    this.initPrivacyNotice();

    this._observer = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
    this._observer.addObserver(this, REPORTER_ALERTS_FORM_SUBMIT_START, false);
    this._observer.addObserver(this, REPORTER_ALERTS_FORM_SUBMIT_PROGRESS, false);
    this._observer.addObserver(this, REPORTER_ALERTS_FORM_SUBMIT_ERROR, false);
    this._observer.addObserver(this, REPORTER_ALERTS_FORM_SUBMIT_END, false);
  },

  /**
   * result: returns true if string begins starts with stringToSearch,
   * otherwise returns false
   */
  startsWith: function(string, stringToSearch)
  {
    return (string.match("^"+stringToSearch)==stringToSearch);
  },

  /**
   * result: shows the "Report a problem" pop-up window, with problem categories
   * to choose from
   */
  showReporterPopUpDialog: function(aEvent)
  {
    //check if we are dealing with a valid page
    if (this.isAllowedReporterURL()) {

      if (content.document.location.href != this.gParamURL) {
        this.resetFormValues();
      }

      //set the current URL
      this.gParamURL = content.document.location.href;
      //hide the identity handler panel
      getIdentityHandler().hide();
      //show the reporter categories pop-up window
      this._reporterCreateReportContainer.hidden = false;
      this._reporterCreateReportContainer.focus = true;
      //center the reporter categories pop-up window
      var rect = this._reporterCreateReportContainer.getBoundingClientRect();
      this._reporterCreateReportContainer.top = (window.innerHeight - rect.height) / 2;
      this._reporterCreateReportContainer.left = (window.innerWidth - rect.width) / 2;
      this._isReporterPopUpShown = true;

      BrowserUI.pushPopup(this, this._reporterCreateReportContainer);
    }
    else {
      //tell the user that the report can't be sent from the current page
      this._observer.notifyObservers(null, REPORTER_ALERTS_FORM_SUBMIT_ERROR, this._stringBundle.getString('reportMessagesErrorNotSuitableURL'));
    }
  },

  /**
   * result: hides the reporter pop-up window
   */
  hideReporterPopUpDialog: function(aEvent)
  {
    this._reporterCreateReportContainer.hidden = true;
    this._isReporterPopUpShown = false;
    BrowserUI.popPopup(this, this._reporterCreateReportContainer);
    return true;
  },

  /**
   * result: opens the privacy policy link in a new tab
   */
  showPrivacyPolicy: function()
  {
    BrowserUI.newTab(this._privacyPolicyURL, true);
    this.hideReporterPopUpDialog();
  },

  /**
   * result: sends the report to the reporter service via POST
   */
  submitReport: function() {
    this.hideReporterPopUpDialog();
    this._reporterCreateReportButton.disabled = true;
    this.sendReport();
  },

  /**
   * result: executed when the user clicks "Cancel" button
   */
  cancelSend: function()
  {
    this.hideReporterPopUpDialog();
  },

  /**
   * result: resets form values
   */
  resetFormValues: function()
  {
    this._reporterComments.value = "";
    this._reporterProblemCategoriesList.selectedIndex = 0;
  },

  /**
   * result: overrides the IdentitiyHandelr.show() function to add the reporter
   * logic for enabling/disabling the "Report a problem" button
   */
  overrideIdentityShow: function()
  {
    this._oldShowFunction = getIdentityHandler().show;
    that = this;

    getIdentityHandler().show = function() {
      that._oldShowFunction.call(getIdentityHandler());
      //check if we are dealing with a valid page
      if (that.isAllowedReporterURL() && !that._isReporterPopUpShown) {
        that._reporterCreateReportButton.disabled = false;
      }
      else {
        that._reporterCreateReportButton.disabled = true;
      }
    }
  },

  /**
   * result: verifies if the report can be sent from the curren location
   */
  isAllowedReporterURL: function()
  {
    currentURL = content.document.location.href;
    if (currentURL != null && currentURL != ""
        && !this.startsWith(currentURL, "about")
        && !this.startsWith(currentURL, "chrome")
        && !this.startsWith(currentURL, "file") ) {
      return true;
    }
    else {
      return false;
    }
  },

  /**
   * result: removes these attributes from the URL:
   * username:password
   * query string
   */
  sanitizeReporterURL: function (paramURL)
  {
    //paramURL = "http://usr:pwd@www.test.com:81/dir/dir.2/index.htm?q1=0&&test1&test2=value#top";
    var sanitizedURL = "";
    var parsedURLArray = parseUri(paramURL);

    if (parsedURLArray["userInfo"] != null &&
        parsedURLArray["userInfo"] != "") {

      userInfoIndex = paramURL.indexOf(parsedURLArray["userInfo"]);

      //remove the userInfo data from the URL: username:password@
      sanitizedURL = paramURL.substring(0, userInfoIndex);
      sanitizedURL += paramURL.substring(userInfoIndex+parsedURLArray["userInfo"].length+1, paramURL.length);
      paramURL = sanitizedURL;
    }

    if (parsedURLArray["query"] != null &&
        parsedURLArray["query"] != "") {

      //remome from the "?" until the end of the URL string
      sanitizedURL = paramURL.substring(0, paramURL.indexOf(parsedURLArray["query"])-1);
    }

    if (sanitizedURL.length > 0) {
      return sanitizedURL;
    }
    else {
      return paramURL;
    }

  },

  /**
   * The following pieces of code are taken from
   * http://mxr.mozilla.org/mozilla-central/source/extensions/reporter/resources/content/reporter/reportWizard.js?raw=1
   * in order to emulate Firefox's reporter mechanism
   */
  getReporterPrefBranch: function()
  {
    try {
      if (!this.gPrefBranch) {
        this.gPrefBranch = Components.classes["@mozilla.org/preferences-service;1"]
                                     .getService(Components.interfaces.nsIPrefService)
                                     .getBranch("extensions.reporter.");
      }
    }
    catch (ex) {
    }
    return this.gPrefBranch;
  },

  getBoolPref: function(prefname, aDefault)
  {
    try {
      var prefs = this.getReporterPrefBranch();

      if (prefs.prefHasUserValue(prefname)) {
        return prefs.getBoolPref(prefname);
      }
      else {
        return aDefault;
      }
    }
    catch(ex) {
      return aDefault;
    }
  },

  getCharPref: function(prefname, aDefault)
  {
    try {
      var prefs = this.getReporterPrefBranch();
      if (prefs.prefHasUserValue(prefname)) {
        return prefs.getCharPref(prefname);
      }
      else {
        return aDefault;
      }
    }
    catch(ex) {
      return aDefault;
    }
  },

  initPrivacyNotice: function()
  {
    // Load Privacy Policy
    this._privacyPolicyURL = this.getCharPref("privacyURL",
                                              "http://reporter.mozilla.org/privacy/")
                              + "?plain";
  },

  sendReport: function()
  {
    // Check for a sysid, if we don't have one, get one and its function will call sendReport on success.
    var sysId = this.getCharPref("sysid", "");
    if (sysId == ""){
      this.registerSysID();
      return;
    }

    // we control the user path from here.
    this.gParamDescription = this._reporterComments.value;
    this.gParamProblemType = this._reporterProblemCategoriesList.selectedItem.value;
    //this.gParamBuildConfig = this.getBuildConfig(); //this method always throws an error
    this.gParamBuildConfig = "Unknown";
    this.gParamGecko = this.getGecko();
    this.gParamURL = this.sanitizeReporterURL(this.gParamURL);

    // params
    var param = {
      'method':           'submitReport',
      'rmoVers':          this.gRMOvers,
      'url':              this.gParamURL,
      'problem_type':     this.gParamProblemType,
      'description':      this.gParamDescription,
      'behind_login':     this.gParamBehindLogin,  //not set
      'platform':         this.gParamPlatform,
      'oscpu':            this.gParamOSCPU,
      'gecko':            this.gParamGecko,
      'product':          this.getProduct(),
      'useragent':        this.gParamUserAgent,
      'buildconfig':      this.gParamBuildConfig,
      'language':         this.gParamLanguage,
      'email':            this.gParamEmail, //not set
      'charset':          this.gCharset,  //not set
      'sysid':            sysId
    };

    this.sendReporterServerData(param, this.onSendReportDataLoad);
  },

  /**
   * result: manages the final report send process
   */
  onSendReportDataLoad: function(req)
  {
    if (req.status != 200) {
      var errorStr = ReporterForFennec.extractError(req);
      ReporterForFennec._observer.notifyObservers(null, REPORTER_ALERTS_FORM_SUBMIT_ERROR, errorStr);
      return;
    }

    var reportId = req.responseXML.getElementsByTagName('reportId').item(0).firstChild.data;
    dump("\nReportId: " + reportId + "\n");
    ReporterForFennec._observer.notifyObservers(null, REPORTER_ALERTS_FORM_SUBMIT_END, reportId);
  },

  registerSysID: function ()
  {
    var param = {
      'method':   'submitRegister',
      'language': this.gParamLanguage
    };
    // go get sysID
    this.sendReporterServerData(param, this.onRegisterSysIDLoad);
  },

  sendReporterServerData: function(params, callback)
  {
    var serviceURL = this.getCharPref("serviceURL", "http://reporter.mozilla.org/service/0.3/");
    params = this.serializeParams(params);
    dump("\nSerialized params = " + params + "\n");

    this._observer.notifyObservers(null, REPORTER_ALERTS_FORM_SUBMIT_START, null);

    var request = new XMLHttpRequest();
    request.onprogress = this.onSendReportDataProgress;
    request.open("POST", serviceURL, true);

    request.onreadystatechange = function () {
      if (request.readyState == 4)
        callback(request);
    };

    request.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    request.setRequestHeader("Content-length", params.length);
    request.setRequestHeader("Connection", "close");
    request.send(params);
  },

  onRegisterSysIDLoad: function(req)
  {
    if (req.status == 200) {
      var paramSysID = req.responseXML.getElementsByTagName('result').item(0);

      // saving
      if (paramSysID) {
        dump("SysId: " + paramSysID.textContent + "\n");
        var prefs = ReporterForFennec.getReporterPrefBranch();
        prefs.setCharPref("sysid", paramSysID.textContent);
        // Finally send report
        ReporterForFennec.sendReport();
        return;
      }

      // Invalid Response Error
      ReporterForFennec._observer.notifyObservers(null, REPORTER_ALERTS_FORM_SUBMIT_ERROR, ReporterForFennec._stringBundle.getString("invalidResponse"));
      return;
    }

    // On error
    var errorStr = ReporterForFennec.extractError(req);
    ReporterForFennec._observer.notifyObservers(null, REPORTER_ALERTS_FORM_SUBMIT_ERROR, errorStr);
  },

  serializeParams: function(params)
  {
    var str = '';
    for (var key in params) {
      str += key + '=' + encodeURIComponent(params[key]) + '&';
    }
    return str.slice(0, -1);
  },

  extractError: function(req)
  {
    var error = req.responseXML.getElementsByTagName('errorString').item(0)
    if (error) {
      return error.textContent;
    }

    // Default error
    return this._stringBundle.getString("defaultError");
  },


  onSendReportDataProgress: function(e)
  {
    dump("\n" + e.postion + " of " + e.totalSize + "\n")
    this._observer.notifyObservers(null, REPORTER_ALERTS_FORM_SUBMIT_PROGRESS, null);
  },

  /**
   *
   */
  observe: function (aSubject, aTopic, aData)
  {
    switch(aTopic)
    {
      case REPORTER_ALERTS_FORM_SUBMIT_START:
        //tell the user that the report can't be sent from the current page
        this._notifier.showAlertNotification(
                            null,
                            this._stringBundle.getString('reportMessagesTitle'),
                            this._stringBundle.getString('reportMessagesSubmit'),
                            false,
                            "",
                            null);
        break;
      case REPORTER_ALERTS_FORM_SUBMIT_PROGRESS:
        /*
        this._notifier.showAlertNotification(
                            null,
                            this._stringBundle.getString('reportMessagesErrorTitle'),
                            this._stringBundle.getString('reportMessagesProgress'),
                            false,
                            "",
                            null);
        */
        break;
      case REPORTER_ALERTS_FORM_SUBMIT_ERROR:
        this._reporterCreateReportButton.disabled = false;
        this._notifier.showAlertNotification(
                            null,
                            this._stringBundle.getString('reportMessagesErrorTitle'),
                            aData,
                            false,
                            "",
                            null);
        break;
      case REPORTER_ALERTS_FORM_SUBMIT_END:
        this._reporterCreateReportButton.disabled = false;
        this._notifier.showAlertNotification(
                            null,
                            this._stringBundle.getString('reportMessagesTitle'),
                            this._stringBundle.getString('reportMessagesReportSent'),
                            false,
                            "",
                            null);
        this.resetFormValues();
        break;
    }
  },


  getBuildConfig: function()
  {
    // bz and Biesi are my heroes for writing/debugging this chunk.
    try {
      var ioservice =
        Components.classes["@mozilla.org/network/io-service;1"]
                  .getService(Components.interfaces.nsIIOService);
      var channel = ioservice.newChannel("chrome://global/content/buildconfig.html", null, null);
      var stream = channel.open();
      var scriptableInputStream =
        Components.classes["@mozilla.org/scriptableinputstream;1"]
                  .createInstance(Components.interfaces.nsIScriptableInputStream);
      scriptableInputStream.init(stream);
      var data = "";
      var curBit = scriptableInputStream.read(4096);
      while (curBit.length) {
        data += curBit;
        curBit = scriptableInputStream.read(4096);
      }
      // Strip out the <!DOCTYPE> part, since it's not valid XML
      data = data.replace(/^<!DOCTYPE[^>]*>/, "");
      // Probably not strictly needed, but what the heck
      data = data.replace(/^<html>/, "<html xmlns='http://www.w3.org/1999/xhtml'>");
      var parser = new DOMParser();
      var buildconfig = parser.parseFromString(data, "application/xhtml+xml");
      var text = buildconfig.getElementsByTagName("body")[0].textContent;
      var start= text.indexOf('Configure arguments')+19;
      return text.substring(start);
    } catch(ex) {
      return "Unknown";
    }
  },

  getProduct: function()
  {
    try {
      // Works on Firefox 1.0+ and Future SeaMonkey's
      var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                              .getService(Components.interfaces.nsIXULAppInfo);
      // Use App info if possible
      return appInfo.name+"/"+appInfo.version;
    }
    catch(ex) {}
    try {
      // only works on Gecko 1.8 and higher (if above doesn't return)
      if ('nsIChromeRegistrySea' in Components.interfaces) {
        return 'SeaMonkey/'+
        Components.classes['@mozilla.org/network/io-service;1']
                  .getService(Components.interfaces.nsIIOService)
                  .getProtocolHandler('http')
                  .QueryInterface(Components.interfaces.nsIHttpProtocolHandler).misc.substring(3);
      }
      // Firefox < 1.0+ or a last ditch effort for others that may fail
      else if (navigator.vendor != ''){
        return window.navigator.vendor+'/'+window.navigator.vendorSub;
      }
      else {
        return "Unknown";
      }
    }
    catch(ex) {
      return "Unknown";
    }
  },

  getGecko: function()
  {
    try {
      var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                              .getService(Components.interfaces.nsIXULAppInfo);
      // Use App info if possible
      return appInfo.platformBuildID;
    }
    catch(ex) {
      return "00000000"; // 8 0's to ignore as we have historically
    }
  }

};


window.addEventListener("load",
                        function () { ReporterForFennec.init() },
                        false);

window.addEventListener("unload",
                        function() { ReporterForFennec.unload() } ,
                        false);
